import { Player, ItemStack, ItemLockMode, EquipmentSlot, } from '@minecraft/server';
// @ts-ignore
export default class CustomPlayer extends Player {
    constructor() {
        super(...arguments);
        this.isInit = false;
    }
    init() {
        this.isInit = true;
    }
    isAlive() {
        const health = this.getComponent('health');
        return health.currentValue > 0;
    }
    setMovement(value) {
        const playerMovementComponent = this.getComponent('movement');
        playerMovementComponent.setCurrentValue(value);
    }
    addMovement(value /* in percent */) {
        const playerMovementComponent = this.getComponent('movement');
        const velocity = playerMovementComponent.defaultValue + (value * playerMovementComponent.defaultValue) / 100;
        this.setMovement(velocity);
    }
    resetMovement() {
        const playerMovementComponent = this.getComponent('movement');
        playerMovementComponent.resetToDefaultValue();
    }
    removeOneHandItem() {
        const item = this.getHandItem();
        if (item.amount <= 1)
            this.setHandItem(null);
        else {
            item.amount -= 1;
            this.setHandItem(item);
        }
    }
    setLockItem(value, slotNum) {
        let item = this.getInventory().getItem(slotNum);
        if (!item)
            return;
        item.lockMode = value ? ItemLockMode.slot : ItemLockMode.none;
        this.getInventory().setItem(slotNum, item);
        return item;
    }
    clearActionBar() {
        this.onScreenDisplay.setActionBar(' ');
    }
    setLockHandItem(value) {
        const item = this.getHandItem();
        item.lockMode = value ? ItemLockMode.slot : ItemLockMode.none;
        this.setHandItem(item);
    }
    getItemSlot(slotNum) {
        return this.getInventory().getItem(slotNum);
    }
    getItemStackSlot(slotNum) {
        return this.getInventory().getItem(slotNum);
    }
    getHandItem() {
        return this.getInventory().getItem(this.selectedSlotIndex);
    }
    clearHandItem() {
        return this.getInventory().setItem(this.selectedSlotIndex, new ItemStack('minecraft:dirt', 1));
    }
    setHandItem(item) {
        this.getInventory().setItem(this.selectedSlotIndex, item);
    }
    getInventory() {
        const inventoryComponent = this.getComponent('inventory');
        return inventoryComponent?.container;
    }
    getOffHandItem() {
        const equipment = this.getComponent('equippable');
        return equipment.getEquipment(EquipmentSlot.Offhand);
    }
    setOffhandItem(item) {
        const equipment = this.getComponent('equippable');
        equipment.setEquipment(EquipmentSlot.Offhand, item);
    }
    clearOffHandItem() {
        const equipment = this.getComponent('equippable');
        equipment.getEquipmentSlot(EquipmentSlot.Offhand).setItem(null);
    }
    isSameLocation(location, tolerance = 0.3) {
        const xDiff = Math.abs(this.location.x - location.x);
        const yDiff = Math.abs(this.location.y - location.y);
        const zDiff = Math.abs(this.location.z - location.z);
        return xDiff <= tolerance && yDiff <= tolerance && zDiff <= tolerance;
    }
    getInventoryItem(itemId) {
        let items = [];
        const playerInventory = this.getInventory();
        for (let i = 0; i < playerInventory.size; i++) {
            const item = playerInventory.getItem(i);
            if (item?.typeId === itemId)
                items.push(item);
        }
        return items;
    }
    getHeight() {
        return this.isSneaking ? 1.25 : 1.5;
    }
}
