import { DimensionTypes, EntityDamageCause, Player, system, world } from '@minecraft/server';
import { Bullet } from 'entities/bullet';
import { Instances } from 'instances/global'
import { DamageHandler } from 'damage_handler';
import { distanceVector } from 'utils/distance-between-vector3';

export default class EventsHandler {

	private readonly onTick: number = system.runInterval(this.tickMethod.bind(this));

	public readonly BULLET_TAG: string = 'bullet';

	constructor() {
		world.afterEvents.projectileHitEntity.subscribe((data) => {
			const { projectile, source: player } = data;

			const hit = data.getEntityHit().entity
			if (!hit || !hit.isValid() || hit.typeId.includes("agbullet:") || hit.typeId.includes("eject:"))
				return;
			let weapon = Instances[projectile.typeId.replace("agbullet:", "")];
			const bullet = projectile as Bullet
			if (projectile.typeId == 'minecraft:arrow')
				weapon = Instances['arrow'];
			if (!weapon || distanceVector(bullet.spawnLocation, hit.location) > weapon.maxRange)
				return;
			if (!bullet.spawnLocation)
				bullet.spawnLocation = player.location

			const damages = DamageHandler.getDamages(weapon, hit, bullet);

			hit.applyDamage(damages, {
				cause: EntityDamageCause.override,
				damagingProjectile: projectile,
				damagingEntity: player
			});
			const direction =  player.getViewDirection();
			const knockbackResistance = DamageHandler.getKnockbackResistance(hit);
			hit.applyKnockback(direction.x, direction.z, (weapon?.knockback?.x * knockbackResistance) || 0, (weapon?.knockback?.y * knockbackResistance) || 0);
			DamageHandler.handleArmorDurability(hit, weapon.damage);

			world.sendMessage("§l§g" + player.nameTag + "§r§e hit §l§g" + hit.nameTag + "§r§e with §l§g" + projectile.typeId.replace("agbullet:", "") + "§r§e for §l§g" + damages + "§r§e damage!")
		});

		world.afterEvents.entityDie.subscribe((ev) => {
			const { damageSource, deadEntity } = ev

			const cause = damageSource.cause
			const killer = damageSource.damagingEntity

			if (cause == EntityDamageCause.override && deadEntity.typeId == "minecraft:player") {
				world.sendMessage({ translate: `death.attack.customBullet`, "with": [deadEntity.nameTag, killer.nameTag] })
			}
		})

		world.afterEvents.entitySpawn.subscribe((ev) => {
			if (!ev.entity || !ev.entity.isValid() || (!ev.entity.typeId.startsWith("agbullet:") && ev.entity.typeId != 'minecraft:arrow'))
				return;
			let weapon = Instances[ev.entity.typeId.replace("agbullet:", "")];
			if (ev.entity.typeId == 'minecraft:arrow')
				weapon = Instances['arrow'];
			if (!weapon)
				return;
			(ev.entity as Bullet).spawnLocation = ev.entity.location;
			(ev.entity as Bullet).maxRange = weapon.maxRange
			ev.entity.addTag(this.BULLET_TAG)
		})
	}

	private tickMethod(): void {
		for (const dim of DimensionTypes.getAll())
			for (const bullet of world.getDimension(dim.typeId).getEntities({ tags: [this.BULLET_TAG] })) {
				if (distanceVector((bullet as Bullet)?.spawnLocation || bullet.location, bullet.location) > ((bullet as Bullet)?.maxRange || 0))
					bullet.remove()
			}
	}
}
